package es.upm.dit.adsw.tema2.balls.bg_task;

import java.awt.*;
import java.util.Date;
import java.util.Random;

public class BgBall_2
        implements Screen.Thing, Runnable {
    private static final Random random = new Random();
    private static final int DELTA = 100;
    private final Screen screen;

    private int cx;
    private int cy;
    private int r = 10;
    private Color body = new Color(getR(), getR(), getR());

    private final int targetX;
    private final int targetY;
    private MyListener listener;
    private String result;

    private int vx;
    private int vy;
    private volatile boolean running;

    public BgBall_2(Screen screen, int targetX, int targetY, MyListener listener) {
        this.screen = screen;
        this.targetX = targetX;
        this.targetY = targetY;
        this.listener = listener;
    }

    private int getR() {
        return 150 + random.nextInt(105);
    }

    public void setColor(Color color) {
        body = color;
    }

    public void setXY(int x0, int y0) {
        cx = x0;
        cy = y0;
    }

    public void setV(int vx, int vy) {
        this.vx = vx;
        this.vy = vy;
    }

    public void quit() {
        running = false;
    }

    @Override
    public void run() {
        running = true;
        long time = System.currentTimeMillis();

        while (running) {
            int width = screen.getWidth();
            int height = screen.getHeight();

            long t = System.currentTimeMillis();
            double dt = (t - time) / 100.0;

            // nueva posición al cabo de un tiempo dt
            int nx = (int) (cx + vx * dt);
            int ny = (int) (cy + vy * dt);
            int nvx = vx;
            int nvy = vy;

            // si se sale del cuadro, rebota
            if (nx < r) {
                nx = r;
                if (nvx < 0)
                    nvx = -nvx;
            } else if (nx > width - r) {
                nx = width - r;
                if (nvx > 0)
                    nvx = -nvx;
            }
            if (ny < r) {
                ny = r;
                if (nvy < 0)
                    nvy = -nvy;
            } else if (ny > height - r) {
                ny = height - r;
                if (nvy > 0)
                    nvy = -nvy;
            }

            // me muevo al nuevo lugar
            setXY(nx, ny);
            setV(nvx, nvy);
            screen.paint();

            int dx = nx - targetX;
            int dy = ny - targetY;
            double d = Math.sqrt(dx * dx + dy * dy);
            if (d < 20)
                break;

            time = System.currentTimeMillis();
            nap(DELTA);
        }

        if (running)
            done();

        screen.remove(this);
        screen.paint();
    }

    private void nap(long ms) {
        try {
            Thread.sleep(ms);
        } catch (InterruptedException ignored) {
        }
    }

    public void paint(Graphics2D g) {
        if (body != null) {
            g.setColor(body);
            g.fillOval(cx - r, cy - r, 2 * r, 2 * r);
        }
        g.setColor(Color.BLACK);
        g.drawOval(cx - r, cy - r, 2 * r, 2 * r);
    }

    private void done() {
        Date date = new Date();
        result = date.toString();
        listener.publish(result);
    }

    public synchronized String getResult() {
        while (result == null)
            waiting();
        return result;
    }

    private void waiting() {
        try {
            wait();
        } catch (InterruptedException ignored) {
        }
    }
}
