package es.upm.dit.adsw.tema2.bounded_buffer;

import java.util.Random;

public class Productor extends Thread {
    private final Random random = new Random();
    private final String id;
    private final Buffer<String> buffer;
    private int n;
    private volatile boolean parado = true;
    private LogWindow logWindow;

    public Productor(String id, Buffer<String> buffer) {
        this.id = id;
        setName(id);
        this.buffer = buffer;
        this.n = 0;
        logWindow = new LogWindow(this);
    }

    @Override
    public void run() {
        try {
            while (true) {
                if (!parado) {
                    String msg = id + ": " + n++;
                    buffer.put(msg);
                    logWindow.println("Produce: " + msg);
                }
                sleep(random.nextInt(5) * 1000);
            }
        } catch (InterruptedException e) {
            logWindow.println(e.toString());
        }
        logWindow.println(id + " acaba");
    }

    public void parar() {
        parado = true;
    }

    public void arrancar() {
        parado = false;
    }
}

