package es.upm.dit.adsw.tema2.varios;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

public class MyBlockingQueue<E> {
    private static final Random random = new Random();

    private final int max;
    private List<E> queue;

    public MyBlockingQueue(int max) {
        this.max = max;
        queue = new ArrayList<E>();
    }

    public synchronized void put(E e) throws InterruptedException {
        while (queue.size() >= max)
            wait();
        queue.add(e);
        notifyAll();
    }

    public synchronized E take() throws InterruptedException {
        try {
            while (queue.isEmpty())
                wait();
            return queue.remove(0);
        } finally {
            notifyAll();
        }
    }

    private static void nap(long ms) {
        try {
            Thread.sleep(ms);
        } catch (InterruptedException ignored) {
        }
    }

    public static void nap(int min, int max) {
        try {
            Thread.sleep(min + random.nextInt(max - min));
        } catch (InterruptedException ignored) {
        }
    }

    // smoke test
    public static void main(String[] args) {
        System.out.println("test 1");
        MyBlockingQueue<String> queue = new MyBlockingQueue<>(3);
        Thread S = new MySender(queue);
        Thread R = new MyReceiver(queue);
        S.start();
        R.start();
    }

    private static class MySender extends Thread {
        private final MyBlockingQueue<String> queue;

        public MySender(MyBlockingQueue<String> queue) {
            this.queue = queue;
        }

        @Override
        public void run() {
            try {
                for (int cnt = 1; cnt < 20; cnt++) {
                    System.out.println("send " + cnt);
                    queue.put("" + cnt);
                    nap(100, 500);
                }
                queue.put(".");
            } catch (InterruptedException ignored) {
            }
        }
    }

    private static class MyReceiver extends Thread {
        private final MyBlockingQueue<String> queue;

        public MyReceiver(MyBlockingQueue<String> queue) {
            this.queue = queue;
        }

        @Override
        public void run() {
            try {
                while (true) {
                    String msg = queue.take();
                    System.out.println("received " + msg);
                    if (msg.equals("."))
                        break;
                    nap(400);
                }
            } catch (InterruptedException ignored) {
            }
        }
    }
}
